package es.jmillandel.senassistant.bbdd;

import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.content.ContentValues;
import android.database.Cursor;
import android.location.Location;
import android.telephony.SmsManager;
import android.util.Log;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Date;
import java.util.List;

import es.jmillandel.senassistant.app.R;
import es.jmillandel.senassistant.beans.Aviso;
import es.jmillandel.senassistant.beans.DatosGenerales;
import es.jmillandel.senassistant.beans.ExceptionApp;
import es.jmillandel.senassistant.beans.Sitio;
import es.jmillandel.senassistant.beans.Trayecto;

/**
 * Clase para gestionar los datos de los trayectos en la BD
 * y otras funciones para determinar la cercania a sitios.
 * 
 * Hereda de GestorBBDD.
 * 
 * @author Juan Millan
 *
 */
public class GestorSeguimiento extends GestorBBDD {

	public GestorSeguimiento(Context contexto, String nombre, CursorFactory factory,
            int version) {
		super(contexto, nombre, factory, version);
	}
	
	/**
	 * Graba un trayecto
	 * 
	 * @param trayecto
	 * @throws ExceptionApp
	 */
	public void setTrayecto(Trayecto trayecto) throws ExceptionApp {
		ContentValues nuevo = new ContentValues();
		nuevo.put("diaHora", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(trayecto.getDiaHora().getTime()));
		nuevo.put("longitud", trayecto.getLongitud());
		nuevo.put("latitud", trayecto.getLatitud());
		nuevo.put("direccion", trayecto.getDireccion());
		nuevo.put("tiempoPermanece", trayecto.getTiempoPerm());
		nuevo.put("tiempoTranscurrido", trayecto.getTiempoTrans());
		nuevo.put("distancia", trayecto.getDistancia());		
		try {
			myDataBase.insert("Trayecto", null, nuevo);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_trayecto_alta));
		}
	}
	
	/**
	 * Borra el trayecto anterior a la fecha actual.
	 * 
	 * @throws ExceptionApp
	 */
	public void bajaTrayectos() throws ExceptionApp {
		String dia=new SimpleDateFormat("yyyy-MM-dd").format(new Date().getTime());
		String diaIni=dia+" 00:00:00";
		
		String sqlText="diaHora < '"+diaIni+"'";
		try {
			myDataBase.delete("Trayecto", sqlText, null);
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_trayecto_baja));
		}
	}
	
	/**
	 * devuelve una lista con los sitios ordenados por posicion.
	 * 
	 * @return List<Sitio> lista con los sitios
	 * @throws ExceptionApp
	 */
	public List<Sitio> listSitios() throws ExceptionApp {
		
		List<Sitio> lstSitios = new ArrayList<Sitio>();
		String sqlText="SELECT * FROM Sitio ORDER BY latitud, longitud";
	
		try {
	        Cursor c = myDataBase.rawQuery(sqlText, null);
	        if (c.moveToFirst()) { 
	            do {
		            Sitio sitio = new Sitio();
		            sitio.setIdSitio(c.getInt(0));    
		        	sitio.setNombre(c.getString(1));
		        	sitio.setDireccion(c.getString(2));
		        	sitio.setLongitud(c.getDouble(3));
		        	sitio.setLatitud(c.getDouble(4));       	 
		            lstSitios.add(sitio);
	            } while(c.moveToNext());
	        }
	        c.close();
		} catch (SQLException e) {
			throw new ExceptionApp(myContext.getString(R.string.error_sitio_lista));
		}
        return lstSitios;     
	}
	
	/**
	 * devuelve el sitio mas cercano a una posicion y de una lista 
	 * de sitios recibidas.
	 * 
	 * @param lat
	 * @param lon
	 * @param lstSitios
	 * @return sitio mas cercano
	 */
	public Sitio getSitioCercano(double lat, double lon, List<Sitio> lstSitios) {
		Sitio sitioResult=new Sitio();
		float dist = 40008000.0f; 
        Location locationDistance=new Location("");
	    float[] locResult=new float[1];
	    
	    if (!lstSitios.isEmpty()) {
			Iterator<Sitio> sitioIterator = lstSitios.iterator();
	        while (sitioIterator.hasNext()) {
	            Sitio sitio = (Sitio)sitioIterator.next();
	            locationDistance.distanceBetween(sitio.getLatitud(),sitio.getLongitud(), lat, lon, locResult);
	    	    
	    	    if (locResult[0]<dist) {
	    	    	sitioResult=sitio;
	    	    	dist=locResult[0];
	    	    }
	        }
		} else {
        	sitioResult=null;
        }
        
        return sitioResult;
	}
	
	/**
	 * devuelve los avisos de un sitio.
	 * 
	 * @param idSitio
	 * @return List<Aviso> lista con los avisos del sitio.
	 * @throws ExceptionApp
	 */
	public List<Aviso> listAvisos(int idSitio) throws ExceptionApp {
		List<Aviso> lstAvisos = new ArrayList<Aviso>();
		
		try{
			GestorSitios gestorSitios= new GestorSitios(myContext,GestorSitios.DB_NAME,null,GestorSitios.DB_VERSION);
	        gestorSitios.open();
	        lstAvisos=gestorSitios.listAvisos(idSitio);
	        gestorSitios.close();
	        return lstAvisos;
        }
        catch (Exception e){
        	throw new ExceptionApp(e.getMessage());
        }
	}
	
	/**
	 * devuelve el telefono SOS de datos generales.
	 * 
	 * @return String con el telefono
	 * @throws ExceptionApp
	 */
	public String getSOSTelefono() throws ExceptionApp {
		DatosGenerales datGen=new DatosGenerales();
		try{
			GestorGeneral gestorGeneral= new GestorGeneral(myContext,GestorGeneral.DB_NAME,null,GestorGeneral.DB_VERSION);
	        gestorGeneral.open();
	        datGen=gestorGeneral.getDatosGen();
	        gestorGeneral.close();
	        return datGen.getContacto();
        }
        catch (ExceptionApp e){
        	throw new ExceptionApp(e.getMessage());
        }
	}
	
	/**
	 * envia un mensaje SMS.
	 * 
	 * @param telefono
	 * @param mensaje
	 */
	public void sendMessage(String telefono, String mensaje) throws ExceptionApp {
		try {
			SmsManager sms = SmsManager.getDefault();
			sms.sendTextMessage(telefono, null, mensaje, null, null);
		} catch (Exception e) {
			throw new ExceptionApp(e.getMessage());
		}
	}
	
}
